@@ -1,4 +1,4 @@
-"""Recipe for setting up a google appengine development environment."""
+"""Recipe for setting up a Google App Engine development environment."""
 
 import logging
 import os
@@ -33,7 +33,9 @@ def copytree(src, dst, symlinks=0):
                 os.symlink(linkto, dstname)
             elif os.path.isdir(srcname):
                 copytree(srcname, dstname, symlinks)
-            else:
+            elif not os.path.isfile(dstname) and symlinks:
+                os.symlink(srcname, dstname)
+            elif not symlinks:
                 shutil.copy2(srcname, dstname)
         except (IOError, os.error), why:
             logging.error("Can't copy %s to %s: %s" % 
@@ -70,7 +72,7 @@ class Zipper(object):
 
 
 class Recipe(zc.recipe.egg.Eggs):
-    """Buildout recipe for google appengine."""
+    """Buildout recipe for Google App Engine."""
 
     def __init__(self, buildout, name, opts):
         """Standard constructor for zc.buildout recipes."""
@@ -113,15 +115,18 @@ class Recipe(zc.recipe.egg.Eggs):
         os.chmod(path, 0755)
 
     def install_appengine(self):
-        """Downloads and installs google appengine."""
+        """Downloads and installs Google App Engine."""
         arch_filename = self.options['url'].split('/')[-1]
         dst = os.path.join(self.buildout['buildout']['parts-directory'])
         downloads_dir = os.path.join(os.getcwd(), 'downloads')
-        if os.path.isdir(downloads_dir):
-            shutil.rmtree(downloads_dir)
-        os.mkdir(downloads_dir)
+        if not os.path.isdir(downloads_dir):
+            os.mkdir(downloads_dir)
         src = os.path.join(downloads_dir, arch_filename)
-        urllib.urlretrieve(self.options['url'], src)
+        if not os.path.isfile(src):
+            logger.info("downloading Google App Engine distribution...")
+            urllib.urlretrieve(self.options['url'], src)
+        else:
+            logger.info("Google App Engine distribution already downloaded.")
         if os.path.isdir(os.path.join(dst, 'google_appengine')):
             shutil.rmtree(os.path.join(dst, 'google_appengine'))
         arch = zipfile.ZipFile(open(src, "rb"))
@@ -165,7 +170,7 @@ class Recipe(zc.recipe.egg.Eggs):
             dir = os.path.join(lib, os.path.basename(src))
             if not os.path.exists(dir):
                 os.mkdir(dir)
-            copytree(src, dir)
+            copytree(src, dir, True)
 
     def zip_packages(self, ws, lib):
         """Creates zip archive of configured packages."""
@@ -204,7 +209,7 @@ class Recipe(zc.recipe.egg.Eggs):
             reqs, ws = self.working_set()
             sources = [d.location for d in ws if d.key in reqs]
         for s in sources:
-            copytree(s, options['app-directory'])
+            copytree(s, options['app-directory'], True)
 
     def install(self):
         """Creates the part."""
